=begin
This script will make an opening in an internal wall group, add an architrave (with material inherited from door component),
and move the door component into the openning by the specified door thickness.

Assumptions:-
* The wall has thickness and this is constant.
* The door component must be glued to a wall group.
* The door component axis origin is the top lefthand corner, rear (ie glued plane).
* The door is rectangular.
* The wall faces (including the underside) within the door area are clean of extraneous geometry.

Instructions:-
1. Place (glue) the internal door component onto the wall (Place on the room side into which the door will open).
2. Highlight the door component or components.
3. Select "db Wall-Int-Open" from the Plugins drop down menu.
4. Select Architrave Type from the drop down list of dialogue box, "Chamfer" or "Bullnose".
5. Click OK and the door openning/architrave will be created and the door component(s) moved into the wall by the door thickness.



Tip:-
To adjust the door/frame/architrave combination position after placement. Edit wall group, pushpull jambs to desired position.
Move door component & frame/architrave group to new position.
=end

module DeanBy7
	module Intdoor_Opening
		def self.main
		    #@olt = 102.5.mm if(!@olt)
			#@twt = 100.mm if(!@twt)
			@dft = 32.mm if(!@dft)
			#@tdt = 35.mm if(!@tdt)
			@arch = "Chamfer" if(!@arch)
			
			mod = Sketchup.active_model
			sel = mod.selection			
			mats = Sketchup.active_model.materials
			sents = mod.entities
			unless sel.grep(Sketchup::ComponentInstance).empty?
				#prompts = ["Architrave Type"]
				#defaults = [@arch]
				#list = ["Chamfer|Bullnose"]
				#results = UI.inputbox prompts, [@arch], list, "db IntDoor Opening"
				#if results
				#@arch = results[0]
				mod.start_operation "db IntDoor Opening"
				sel.grep(Sketchup::ComponentInstance){|cdi|
					trn = cdi.transformation
	 		    	org = cdi.transformation.origin
					wid = cdi.definition.bounds.width
					cents = cdi.definition.entities
					vert = nil
					cents.grep(Sketchup::Edge){|e|
						e.vertices.each{|v| vpt=v.position.transform(trn)
						vert = v if vpt == org; }
					}
					unless vert==nil
						tdt = 1e9
						vert.edges.each{|e|
							tdt = [tdt,e.length].min
						}
						#puts "Thickness of #{cdi.definition.name} is #{tdt}"
					end
					hgt = [cdi.definition.bounds.depth,cdi.definition.bounds.height].max
					fog = cdi.glued_to
					fmat = cdi.material
					if fog!=nil
						if fog.is_a?(Sketchup::Face)
							#p0 = org; ents = mod.active_entities
							#p "face"
							#p p0
							#xa,ya,za = fog.normal.axes
							#omat = fog.material
							UI.messagebox "Wall not a group", MB_OK
						elsif fog.is_a?(Sketchup::Group)
							gt = fog.transformation
							ents = fog.entities
							p0 = org.transform(gt.inverse)
							#p "group"
							#p p0
							ptest = p0 + [0,0,-1]
							gf = ents.grep(Sketchup::Face).find{|f|f.classify_point(ptest)==1}
							xa,ya,za = gf.normal.axes
							omat = gf.material
						end
						cent = gf.bounds.center
						gfnormal = gf.normal
						vec = gfnormal.reverse
						gfplane = gf.plane
						ofaces = []
						ents.grep(Sketchup::Face).each{|e| ofaces << e if e.normal == vec }
							#p "All backside faces"
							#p ofaces.length
							#ofaces.each{|f| f.reverse!} #to check which faces selected
							nfaces = []
							ofaces.each{|face|
							pt=Geom.intersect_line_plane([cent,gfnormal],face.plane)
							#ents.add_line(cent,pt)
							fcp=face.classify_point(pt)
							if fcp == 1
								nfaces << face
							end#if
						}
						twt = 0
						nfaces.each{|face|
							pt=Geom.intersect_line_plane([cent,gfnormal],face.plane)
							#p cent.distance(pt)
							if (cent.distance(pt) < 500.mm)
								twt = cent.distance(pt)
							end#if
						}
				
						cdi.glued_to = nil
						p0.offset!(ya,-hgt)
						p1 = p0.offset(xa,wid + @dft) 
						p2 = p1.offset(ya,hgt + @dft)
						p4 = p0.offset(xa,-@dft)
						p3 = p4.offset(ya,hgt + @dft)
						p0.offset!(za,-twt)
						p1a = p0.offset(xa,wid + @dft)
						p2a = p1a.offset(ya,hgt + @dft)
						p4a = p0.offset(xa,-@dft)
						p3a = p4a.offset(ya,hgt + @dft)
						botface = ents.add_face(p1,p1a,p4a,p4)
						botface.pushpull(-hgt - @dft)
						ents = mod.active_entities
						frm_grp = ents.add_group()
						frm_grp.name = "Frame"
						fents = frm_grp.entities
						line1 = fents.add_line(p1,p2)
						line2 = fents.add_line(p2,p3)
						line3 = fents.add_line(p3,p4)
						lines = [line1, line2, line3]
						p1b = p1a.offset(xa,-@dft)
						p1c = p1.offset(xa,-@dft)
						pd4 = p1c.offset(za,-tdt)
						pd3 = pd4.offset(xa,-12.mm)
						pd2 = pd3.offset(za,-32.mm)
						pd1 = pd4.offset(za,-32.mm)
						dfr = fents.add_face(p1,p1a,p1b,pd1,pd2,pd3,pd4,p1c)
						dfr.followme(lines)
						fents = frm_grp.entities
						line1 = fents.add_line(p1,p2)
						line2 = fents.add_line(p2,p3)
						line3 = fents.add_line(p3,p4)
						lines = [line1, line2, line3]
						if (@arch == "Chamfer")
							pa1 = p1c.offset(xa,12.mm)
							pa2 = pa1.offset(xa,69.mm)
							pa3 = pa2.offset(za,20.mm)
							pa4 = pa3.offset(xa,-20.mm)
							pa5 = pa1.offset(za,10.mm)
							pa5.offset!(xa,5.mm)
							aface = fents.add_face(pa1,pa2,pa3,pa4,pa5)
						elsif (@arch == "Bullnose")
							pa1 = p1c.offset(xa,12.mm)
							pa2 = pa1.offset(xa,69.mm)
							pa3 = pa2.offset(za,20.mm)
							#pa4 = pa3.offset(xa,-20.mm)
							pa5 = pa1.offset(za,20.mm)
							#pa5.offset!(xa,5.mm)
							aface = fents.add_face(pa1,pa2,pa3,pa5)
						end
						aface.followme(lines)
						line1 = fents.add_line(p1a,p2a)
						line2 = fents.add_line(p2a,p3a)
						line3 = fents.add_line(p3a,p4a)
						lines = [line1, line2, line3]
						if (@arch == "Chamfer")
							paa1 = p1b.offset(xa,12.mm)
							paa2 = paa1.offset(xa,69.mm)
							paa3 = paa2.offset(za,-20.mm)
							paa4 = paa3.offset(xa,-20.mm)
							paa5 = paa1.offset(za,-10.mm)
							paa5.offset!(xa,5.mm)
							aaface = fents.add_face(paa1,paa2,paa3,paa4,paa5)
						elsif (@arch == "Bullnose")
							paa1 = p1b.offset(xa,12.mm)
							paa2 = paa1.offset(xa,69.mm)
							paa3 = paa2.offset(za,-20.mm)
							#paa4 = paa3.offset(xa,-20.mm)
							paa5 = paa1.offset(za,-20.mm)
							#paa5.offset!(xa,5.mm)
							aaface = fents.add_face(paa1,paa2,paa3,paa5)
						end
						aaface.followme(lines)
						frm_grp.material = fmat
						if fog.is_a?(Sketchup::Group)
							frm_grp.transform!(gt)
						end
						
						vect = cdi.transformation.zaxis.reverse
						vect.length = tdt
						cdi.transform!(vect)
					else
						UI.messagebox "Component not Glued!", MB_OK					  
					end # if fog
				} #sel
				mod.commit_operation
				#end #if results
			else
				UI.messagebox "No door components selected"			
			end #unless
		end #def
	end #module
end #module

unless file_loaded?("db4internaldooropen")
	UI.menu("Plugins").add_item("db Wall-Int-Open") { DeanBy7::Intdoor_Opening.main }
	file_loaded("db4internaldooropen")
end
